using System;
using System.ComponentModel;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Specialized;
using System.IO;
using System.Reflection;


namespace EditorControl
{	
	public class HtmlEditor : Table, INamingContainer
	{
		static string startupscript;
		private HtmlGenericControl workPlace;
		private HtmlInputHidden postedDoc;
		private HtmlGenericControl ctrl;
		private string _htmlDoc;

		private bool IsIE55()
		{
			double dblVer=Page.Request.Browser.MajorVersion + Page.Request.Browser.MinorVersion;
			return (Page.Request.Browser.Browser == "IE" && dblVer >= 5.5);
		}

		static HtmlEditor()
		{
			Assembly asm = Assembly.GetExecutingAssembly();

			if(asm != null)
			{
				string resource = "EditorControl.HtmlEditor.js";
	
				Stream stm = asm.GetManifestResourceStream(resource);
				StreamReader reader = new StreamReader(stm);
			
				startupscript = reader.ReadToEnd();
				reader.Close();
				stm.Close();
			}
		}

		public HtmlEditor()
		{
			this.BorderWidth = 0;
			this.CellPadding = 0;
			this.CellSpacing = 0;
			this.Attributes["bgcolor"] = "buttonface";
		}

		public event EventHandler Submit;
            
		protected virtual void OnSubmit()
		{
			if (Submit != null)
			{
				Submit(this,EventArgs.Empty);
			}
		}

		private void btnSubmit_Clicked(object sender, EventArgs e)
		{
			InnerHTML = postedDoc.Value;
			OnSubmit();
		}


		protected override void CreateChildControls()
		{
			base.Controls.Clear();

			string folderPath = Page.Server.MapPath("HtmlEditor");

			if(!Directory.Exists(folderPath))
			{
				try
				{
					Directory.CreateDirectory(folderPath);
				}
				catch(Exception ex)
				{
					string errorMsg = ex.ToString();
				}
			}
			
			//================== P ===================
			TableRow row = new TableRow();
			TableCell tc = new TableCell();

			HtmlSelect select = new HtmlSelect();
			ListItem list = new ListItem("Verdana","Verdana");
			select.Items.Add(list);
			list = new ListItem("Arial","Arial");
			select.Items.Add(list);
			list = new ListItem("Comic Sans MS", "Comic Sans MS");
			select.Items.Add(list);
			list = new ListItem("Courier New","Courier New");
			select.Items.Add(list);
			list = new ListItem("Lucida Console","Lucida Console");
			select.Items.Add(list);
			list = new ListItem("MS Sans Serif","MS Sans Serif");
			select.Items.Add(list);
			list = new ListItem("Tahoma","Tahoma");
			select.Items.Add(list);
			list = new ListItem("Times New Roman","Times New Roman");
			select.Items.Add(list);
			list = new ListItem("Trebuchet MS","Trebuchet MS");
			select.Items.Add(list);
			list = new ListItem("Wingdings","Wingdings");
			select.Items.Add(list);
			select.ID = "fontName";
			select.Attributes["onchange"] = "ChangeFFormat(this,'FontName');";
			select.SelectedIndex = 1;
			tc.Controls.Add(select);

			select = new HtmlSelect();
			list = new ListItem("xx-small","1");
			select.Items.Add(list);
			list = new ListItem("x-small","2");
			select.Items.Add(list);
			list = new ListItem("small","3");
			select.Items.Add(list);
			list = new ListItem("medium","4");
			select.Items.Add(list);
			list = new ListItem("large","5");
			select.Items.Add(list);
			list = new ListItem("x-large","6");
			select.Items.Add(list);
			list = new ListItem("xx-large","7");
			select.Items.Add(list);
			select.ID = "fontSize";
			select.Attributes["onchange"] = "ChangeFFormat(this,'FontSize');";
			select.SelectedIndex = 2;
			tc.Controls.Add(select);

			Setctrl("ChangeStyle(this,'Bold');", "EditorControl.Bold.gif", "Bold", "btnBold");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Italic');", "EditorControl.Italic.gif", "Italic", "btnItalic");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Underline');", "EditorControl.Underline.gif", "Underline", "btnUnderline");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Justifyleft');", "EditorControl.Left.gif", "Justifyleft", "btnJustLeft");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Justifycenter');", "EditorControl.Center.gif", "Justifycenter", "btnJustCenter");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Justifyright');", "EditorControl.Right.gif", "Justifyright", "btnJustRight");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Insertunorderedlist');", "EditorControl.UnorderedList.gif", "UnorderedList", "btnUOrderList");
			tc.Controls.Add(ctrl);

			Setctrl("ChangeStyle(this,'Insertorderedlist');", "EditorControl.OrderedList.gif", "OrderedList", "btnOrderList");
			tc.Controls.Add(ctrl);

			postedDoc = new HtmlInputHidden();
			postedDoc.Value = "";
			postedDoc.ID = "postedDoc";
			tc.Controls.Add(postedDoc);

			row.Controls.Add(tc);
			base.Rows.Add(row);

			//================== Q ===================
			row = new TableRow();
			tc = new TableCell();

			workPlace = new HtmlGenericControl("DIV");
			workPlace.Style["OVERFLOW"] = "auto";
			workPlace.Style["HEIGHT"] = (this.Height).ToString();
			workPlace.Style["WIDTH"] = (this.Width).ToString();
			workPlace.Style["BACKGROUND-COLOR"] = "white";
			workPlace.Style["BORDER"] = "1px inset buttonface";
			workPlace.Style["PADDING"] = "5px";
			workPlace.Attributes["contenteditable"] = "true";
			workPlace.ID = "doc";
			
			tc.Controls.Add(workPlace);

			row.Controls.Add(tc);
			base.Rows.Add(row);

			//================== R ===================
			row = new TableRow();
			tc = new TableCell();

			Button btnSubmit = new Button();
			btnSubmit.ID = "btnSubmit";
			btnSubmit.Click += new EventHandler(this.btnSubmit_Clicked);
			btnSubmit.Attributes.Add("onClick","return SaveDoc(this);");
			btnSubmit.CausesValidation = false;

			btnSubmit.Text = "o";
			btnSubmit.Style["cursor"] = "hand";
			tc.Controls.Add(btnSubmit);

			row.Controls.Add(tc);
			base.Rows.Add(row);
		}

		private void Setctrl(string command, string imgUrl, string toolTip, string id)
		{
			ctrl = new HtmlGenericControl("A");
			ctrl.Attributes["onclick"] = command;
			ctrl.ID = id;
			ctrl.Attributes["href"] = "#";
			EditorImage editorImage = new EditorImage(imgUrl);
			editorImage.ToolTip = toolTip;
			editorImage.Style["VERTICAL-ALIGN"] = "bottom"; 
			editorImage.Style["BORDER"] = "1px solid buttonface";
			editorImage.Style["BACKGROUND-COLOR"] = "#DCDAE6";
			editorImage.Style["CURSOR"] = "hand";
			ctrl.Controls.Add(editorImage);
		}

		protected override void OnPreRender(EventArgs e)
		{
			if (!Page.IsClientScriptBlockRegistered("HtmlEditorScript"))
				Page.RegisterClientScriptBlock("HtmlEditorScript", startupscript);

			string init = String.Format(
				"new HtmlEditor('{0}', '{1}')", 
				this.ClientID, 
				this.IsIE55().ToString().ToLower());
			Page.RegisterArrayDeclaration("editors", init);

			workPlace.InnerHtml = InnerHTML;
			
			base.OnLoad(e);
		}

		protected override void Render(HtmlTextWriter output)
		{
			if (IsIE55())
			{
				EnsureChildControls();
				base.Render(output);
			}
		}

		[Category("Layout")]
		public override Unit Height
		{
			get { return base.Height; }
			set 
			{ 
				base.Height = value;
				ChildControlsCreated = false;
			}
		}

		[Category("Layout")]
		public override Unit Width
		{
			get { return base.Width; }
			set 
			{ 
				base.Width = value;
				ChildControlsCreated = false;
			}
		}

		[Category("Data")]
		public string InnerHTML
		{
			get{ return _htmlDoc; }
			set{ _htmlDoc = value; }
		}
	}
}
